package no.hig.okolloen.graphics;

import java.awt.*;

/**
 * Dial is a class used to produce a GUI component that resembles a cars speed dial.
 * The Dial class builds on the Panel class so you can use it as such. 
 * Updating the Dial component happens off screen so you should not be bothered with 
 * screen flicker.
 * You can set various options for the Dial component like :
 * <ul>
 * <li>The number of major ticks (5 or 6)
 * <li>The color of all minor tick intervals (default to white)
 * <li>the minimum and maximum value
 * <li>The value showed on the dial
 * </ul>
 * <p>
 * The dial component is freely scalable in both directions.
 * 
 * @author      Oivind Kolloen
 * @version     %I%, %G%
 * @since       rev 0.1
 */
public class Dial extends Panel {
    private transient Graphics bg;
    private transient Image buffer;
    private transient int bufferWidth; 
    private transient int bufferHeight;
    private Color colors[] = new Color[20];
    private String labels[] = new String[6];
    private int dividers = 6;
    private double minValue = 0;
    private double maxValue = 100;
    private double currentValue = -1;

    /**
     *	Default constructor for Dial class.
     */
    public Dial () {
        super ();
        for (int i=0; i<20; i++)
            colors[i] = Color.white;
        for (int i=0; i<6; i++)
            setLabel (i+1," "+i*100);
    }

    /**
     *	Sets the current value.
     *  @param current the new current value.
     */
    public void setValue (double value) {
        if (bg!=null) {
            removeDial (bg, getSize().height, getSize().width, currentValue);
            currentValue = value;
            drawDial (bg, getSize().height, getSize().width);
            Graphics g = getGraphics();
            g.drawImage(buffer, 0, 0, this);
            g.dispose();
        } else
            currentValue = value;
    }

    /**
     *	Sets the minimum value allowed.
     *  @param min the new minimum value.
     */
    public void setMinValue (double min) {
        minValue = min;
        setValue (currentValue);
    }

    /**
     *	Sets the maximum value allowed.
     *  @param max the new maximum value.
     */
    public void setMaxValue (double max) {
        maxValue = max;
        setValue (currentValue);
    }

    /**
     *	Sets the color of the "element" part of the dial.
     *  @param element the element to change and color is the color that is wanted.
     *  color can be any of the following : White, Red, Green, Yellow, Orange, Blue.
     */
    public void setColor (int element, String color) {
        if (dividers==6)
            element = 19-element;
        else 
            element = 15-element;
        if (color.equals("White"))
            colors[element] = Color.white;
        if (color.equals("Red"))
            colors[element] = Color.red;
        if (color.equals("Green"))
            colors[element] = Color.green;
        if (color.equals("Yellow"))
            colors[element] = Color.yellow;
        if (color.equals("Orange"))
            colors[element] = Color.orange;
        if (color.equals("Blue"))
            colors[element] = Color.blue;
    }

    /**
     *	Sets the label of the "labelNr" tick on the scale to labelTxt.
     *  @param labelNr is the tick to change, labelTxt is the text for the label.
     */
    public void setLabel (int labelNr, String labelTxt) {
        labels[dividers-labelNr] = labelTxt;
    }

    /**
     *	Updates this component.
     *      @param g the graphics context.
     *	@see java.awt.Component#update
     */
    public void update(Graphics g) {
        paint(g);
    }

    /**
     *	Paints this component.
     *	@param g the graphics context.
     *	@see java.awt.Component#paint
     */	
    public void paint(Graphics g) {
        int height = getSize().height;
        int width = getSize().width;

        if(buffer == null || width != bufferWidth || height != bufferHeight) {
            buffer = createImage(width, height);
            bg = buffer.getGraphics();
            bufferWidth = width;
            bufferHeight = height;
        }

        bg.setColor (getBackground());
        bg.fillRect (0, 0, width, height);
        bg.setColor (Color.black);
        bg.fillArc (24, 24, width-48, height*2-48, 10, 160);
        colorFills (bg, height, width, dividers);
        bg.setColor (Color.black);
        marks (bg, height, width, dividers);
        bg.fillArc (35, 35, width-70, height*2-70, 10, 160);
        printLabels (bg, height, width, dividers);
        bg.setColor (getBackground());
        bg.fillArc (37, 37, width-74, height*2-74, 0, 180);
        bg.setColor (Color.black);
        if ((currentValue <= maxValue)&&(currentValue >= minValue))
            drawDial (bg, height, width);
        Rectangle clip = g.getClipBounds();
        if(clip != null)
            g.setClip(clip);

        g.drawImage(buffer, 0, 0, this);
        g.dispose();
    }

    /**
     *	Returns the preferred size of this component.
     *	@return A Dimension object representing the preferred size of this component.
     *	@see java.awt.Component#getPreferredSize
     */
    public Dimension getPreferredSize() {
        return new Dimension(200, 125);
    }

/********************************************************************************/
/* Only private functions below this point, nothing you will want to mess with. */
/********************************************************************************/

    private void removeDial (Graphics g, int height, int width, double currentValue) {
        bg.setColor (getBackground());
        g.fillOval (width/2-2, height-10, 4, 4);
        if ((currentValue>maxValue)||(currentValue<minValue))
            return;
        int now = (int)((currentValue/(maxValue-minValue))*(double)160);
        now = (180-10)-now;

        double xOffs = (double)(width)/(double)2;
        double xAngleValue = java.lang.Math.cos (java.lang.Math.toRadians(now));
        double xSize = ((double)(width-80)/(double)2);
        double yOffs = (double)(height*2)/(double)2;
        double yAngleValue = java.lang.Math.sin (java.lang.Math.toRadians(now));
        double ySize = ((double)(height*2-80)/(double)2);

        int tipXCoord = ((int)(xOffs+xAngleValue*xSize));
        int tipYCoord = ((int)(yOffs-yAngleValue*ySize));
        if ((now<40)||(now>140)) {
            g.drawLine (width/2, height-8, tipXCoord, tipYCoord);
            g.drawLine (width/2, height-11, tipXCoord, tipYCoord);
        } else {
            g.drawLine (width/2-2, height-10, tipXCoord, tipYCoord);
            g.drawLine (width/2+1, height-10, tipXCoord, tipYCoord);
        }
    }

    private void drawDial (Graphics g, int height, int width) {
        g.setColor (Color.black);
        g.fillOval (width/2-2, height-10, 4, 4);
        if ((currentValue>maxValue)||(currentValue<minValue))
            return;
        int now = (int)((currentValue/(maxValue-minValue))*(double)160);
        now = (180-10)-now;

        double xOffs = (double)(width)/(double)2;
        double xAngleValue = java.lang.Math.cos (java.lang.Math.toRadians(now));
        double xSize = ((double)(width-80)/(double)2);
        double yOffs = (double)(height*2)/(double)2;
        double yAngleValue = java.lang.Math.sin (java.lang.Math.toRadians(now));
        double ySize = ((double)(height*2-80)/(double)2);

        int tipXCoord = ((int)(xOffs+xAngleValue*xSize));
        int tipYCoord = ((int)(yOffs-yAngleValue*ySize));
        if ((now<40)||(now>140)) {
            g.drawLine (width/2, height-8, tipXCoord, tipYCoord);
            g.drawLine (width/2, height-11, tipXCoord, tipYCoord);
        } else {
            g.drawLine (width/2-2, height-10, tipXCoord, tipYCoord);
            g.drawLine (width/2+1, height-10, tipXCoord, tipYCoord);
        }
    }

    private void marks (Graphics g, int height, int width, int number) {
        if (number==5)
            for (int i=10; i<=170; i += 40) {
                bg.fillArc (20, 20, width-40, height*2-40, i, 1);
                if (i<160)
                    for (int ii=10; ii<40; ii += 10)
                        bg.fillArc (25, 25, width-50, height*2-50, ii+i, 1);
            }
        else if (number==6)
            for (int i=10; i<=170; i += 32) {
                bg.fillArc (20, 20, width-40, height*2-40, i, 1);
                if (i<160)
                    for (int ii=8; ii<30; ii += 8)
                        bg.fillArc (25, 25, width-50, height*2-50, ii+i, 1);
            }
    }

    private void colorFills (Graphics g, int height, int width, int number) {
        if (number==5)
            for (int i=0; i<16; i++) {
                bg.setColor (colors[i]);
                bg.fillArc (26, 26, width-52, height*2-52, i*10+10, 10);
            }
        else if (number==6)
            for (int i=0; i<20; i++) {
                bg.setColor (colors[i]);
                bg.fillArc (26, 26, width-52, height*2-52, i*8+10, 8);
            }
    }

    private void printLabels (Graphics g, int height, int width, int number) {
        FontMetrics fm = g.getFontMetrics();
        int labelWidth = fm.stringWidth (labels[0]);
        if (number==6) {
            g.drawString (labels[0], labelXCoord (width-50, 0, number)+27, labelYCoord(height*2-50, 0, number)+25+fm.getHeight()/2);
            g.drawString (labels[1], labelXCoord (width-50, 1, number)+27, labelYCoord(height*2-50, 1, number)+25);
            g.drawString (labels[2], labelXCoord (width-50, 2, number)+25-fm.stringWidth(labels[2])/2, labelYCoord(height*2-50, 2, number)+20);
            g.drawString (labels[3], labelXCoord (width-50, 3, number)+23-fm.stringWidth(labels[3])/2, labelYCoord(height*2-50, 3, number)+20);
            g.drawString (labels[4], labelXCoord (width-50, 4, number)+20-fm.stringWidth(labels[4]), labelYCoord(height*2-50, 4, number)+25);
            g.drawString (labels[5], labelXCoord (width-50, 5, number)+20-fm.stringWidth(labels[5]), labelYCoord(height*2-50, 5, number)+25+fm.getHeight()/2);
        } else
        for (int i=0; i<number; i++) {
            g.setColor (Color.black);
            g.drawString (labels[i], labelXCoord (width-50, i, number)+25, labelYCoord(height*2-50, i, number)+25);
            g.setColor (Color.red);
            g.fillOval (labelXCoord (width-50, i, number)+23, labelYCoord(height*2-50, i, number)+23, 4, 4); 
        }

    }

    private int labelXCoord (int width, int number, int total) {
        int degrees[] = new int[6];
        if (total == 5)
            for (int i=0; i<5; i++)
                degrees[i]=180-(i*40+10);
        else 
            for (int i=0; i<6; i++)
                degrees[i]=180-(i*32+10);
        return (int)((double)width/(double)2-(java.lang.Math.cos(java.lang.Math.toRadians(degrees[number]))*((double)width/(double)2)));
    }

    private int labelYCoord (int height, int number, int total) {
        int degrees[] = new int[6];
        if (total == 5)
            for (int i=0; i<5; i++)
                degrees[i]=180-(i*40+10);
        else 
            for (int i=0; i<6; i++)
                degrees[i]=180-(i*32+10);
        return (int)((double)height/(double)2-(java.lang.Math.sin(java.lang.Math.toRadians(degrees[number]))*((double)height/(double)2)));
    }
}