package no.hig.okolloen.graphics;

import java.awt.*;
import java.awt.event.MouseEvent;
import javax.swing.event.MouseInputAdapter;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.util.Vector;

/**
 * DragControl is a class used to represent a drag controller much like those you will
 * find on an audio mixing table. 
 * You can set various options for the DragControl component like :
 * <ul>
 * <li>the minimum and maximum value
 * <li>The value currently represented
 * </ul>
 * The DragControl also implements the Adjustable interface in the same way as 
 * standard Java scrollbars. This means that you can use a regular adjustmentListener
 * to monitor this component. Note that the DragControl supports decimal numbers
 * something the Adjustable interface does not support so the values returned when
 * using function from this part will return truncated values.
 * Look at java.awt.event.AdjustmentListener for more about this.
 * 
 * @author      Oivind Kolloen
 * @version     %I%, %G%
 * @since       rev 0.1
 */
public class DragControl extends Panel implements Adjustable {
    private transient Graphics bg;
    private transient Image buffer;
    private transient int bufferWidth; 
    private transient int bufferHeight;
    private double minValue = 0;
    private double maxValue = 100;
    private double currentValue = 0;
    private MyListener myListener = new MyListener ();
    private Vector adjustmentListeners = new Vector ();

    /**
     *	Default constructor for DragControl class.
     */
    public DragControl () {
        super ();
        addMouseListener(myListener);
        addMouseMotionListener(myListener);
    }

    /**
     *	Sets the current value.
     *  @param current the new current value.
     */
    public void setValue (double value) {
        if (bg!=null) {
            currentValue = value;
            Graphics g = getGraphics();
            g.drawImage(buffer, 0, 0, this);
            g.dispose();
        } else
            currentValue = value;
    }

    /**
     *	Sets the minimum value allowed.
     *  @param min the new minimum value.
     */
    public void setMinValue (double min) {
        minValue = min;
        setValue (currentValue);
    }

    /**
     *	Sets the maximum value allowed.
     *  @param max the new maximum value.
     */
    public void setMaxValue (double max) {
        maxValue = max;
        setValue (currentValue);
    }

    /**
     *	Updates this component.
     *      @param g the graphics context.
     *	@see java.awt.Component#update
     */
    public void update(Graphics g) {
        paint(g);
    }

    /**
     *	Paints this component.
     *	@param g the graphics context.
     *	@see java.awt.Component#paint
     */	
    public void paint(Graphics g) {
        int height = getSize().height;
        int width = getSize().width;

        if(buffer == null || width != bufferWidth || height != bufferHeight) {
            buffer = createImage(width, height);
            bg = buffer.getGraphics();
            bufferWidth = width;
            bufferHeight = height;
        }

        bg.setColor (getBackground());
        bg.fillRect (0, 0, width, height);
        bg.setColor (Color.black);
        bg.fillRect (width/2-2, 4, 4, height-8);
        drawMarks (bg, width, height);
        int yPos = (int)((height-18)-currentValue/(maxValue-minValue)*(height-18));
        drawControl (bg, width-8, 16, 4, yPos);

        Rectangle clip = g.getClipBounds();
        if(clip != null)
            g.setClip(clip);

        g.drawImage(buffer, 0, 0, this);
        g.dispose();
    }

    /**
     *	Returns the preferred size of this component.
     *	@return A Dimension object representing the preferred size of this component.
     *	@see java.awt.Component#getPreferredSize
     */
    public Dimension getPreferredSize() {
        return new Dimension(30, 125);
    }

    /**
     *  Gets the orientation of this component, will always return DragControl.VERTICAL.
     */
    public int getOrientation() {
        return VERTICAL;
    }

    /**
     *  Sets the minimum value of the DragControl object.
     *  @param min the minimum value
     */
    public void setMinimum(int min) {
        setMinValue ((double)min);
    }

    /**
     *  Gets the minimum value of the DragControl object. (truncated)
     */
    public int getMinimum() {
        return (int)minValue;
    }
    
    /**
     *  Sets the maximum value of the DragControl object.
     *  @param min the minimum value
     */
    public void setMaximum(int max) {
        setMaxValue ((double)max);
    }

    /**
     *  Gets the maximum value of the DragControl object. (truncated)
     */
    public int getMaximum() {
        return (int)maxValue;
    }

    /**
     *  Dummy to satisfy the Adjustable interface
     */
    public void setUnitIncrement(int u) {
    }

    /**
     *  Dummy to satisfy the Adjustable interface, always returns 1
     */
    public int getUnitIncrement() {
        return 1;
    }

    /**
     *  Dummy to satisfy the Adjustable interface
     */
    public void setBlockIncrement(int b) {
    }

    /**
     *  Dummy to satisfy the Adjustable interface, always returns 1
     */
    public int getBlockIncrement() {
        return 1;
    }

    /**
     *  Dummy to satisfy the Adjustable interface
     */
    public void setVisibleAmount(int v) {
    }

    /**
     *  Dummy to satisfy the Adjustable interface, always returns 1
     */
    public int getVisibleAmount() {
        return 1;
    }

    /**
     *  Sets the current value of the DragControl object.
     *  @param v the new value
     */
    public void setValue(int v) {
        setValue ((double)v);
    }

    /**
     *  Gets the current value of the DragControl object. (truncated)
     */
    public int getValue() {
        return (int)currentValue;
    }

    /**
     *  Add a listener to recieve adjustment events when the value of 
     *  the adjustable object changes.
     *  @param l the listener to recieve events
     *	@see java.awt.event.AdjustmentEvent
     */
    public void addAdjustmentListener (AdjustmentListener l) {
        adjustmentListeners.add (l);
    }

    /**
     *  Removes an adjustment listener.
     *  @param l the listener being removed
     *	@see java.awt.event.AdjustmentEvent
     */
    public void removeAdjustmentListener(AdjustmentListener l) {
        adjustmentListeners.removeElement (l);
    }

    private void drawControl (Graphics g, int width, int height, int xOffs, int yOffs) {
        g.setColor (Color.white);
        // Upper and left border
        g.drawLine (xOffs, yOffs, xOffs+width-1, yOffs);
        g.drawLine (xOffs, yOffs, xOffs, yOffs+height);
        g.drawLine (xOffs, yOffs+1, xOffs+width-2, yOffs+1);
        g.drawLine (xOffs+1,yOffs, xOffs+1, yOffs+height-1);
        // Fill inner part
        g.fillRect (xOffs+2, yOffs+2, width-4, height-4);
        g.setColor (Color.black);
        // Bottom and right
        g.drawLine (xOffs+1, yOffs+height, xOffs+width, yOffs+height);
        g.drawLine (xOffs+width, yOffs, xOffs+width, yOffs+height);
        // Inner part
        g.drawLine (xOffs+2, yOffs+2, xOffs+2, yOffs+height-2);
        g.drawLine (xOffs+width-2, yOffs+2, xOffs+width-2, yOffs+height-2);
        for (int i=2; i<height-1; i+=2)
            g.drawLine (xOffs+3, yOffs+i, xOffs+width-3, yOffs+i);
        g.setColor (new Color (128, 128, 128));
        // Bottom and right
        g.drawLine (xOffs+2, yOffs+height-1, xOffs+width-1, yOffs+height-1);
        g.drawLine (xOffs+width-1, yOffs+1, xOffs+width-1, yOffs+height-1);
    }

    private void drawMarks (Graphics g, int width, int height) {
        int marks = (height-10)/12;
        int lead = (height-marks*12)-5;
        g.setColor (Color.lightGray);
        for (int i=0; i<marks+1; i++) {
            g.drawLine (2, lead+i*12-1, width-2, lead+i*12-1);
            g.drawLine (2, lead+i*12, width-2, lead+i*12);
        }
    }

    class MyListener extends MouseInputAdapter {
        public void mousePressed(MouseEvent e) {
            updatePos(e);
        }

        public void mouseDragged(MouseEvent e) {
            updatePos(e);
        }

        public void mouseReleased(MouseEvent e) {
            updatePos(e);
        }

        void updatePos(MouseEvent e) {
            int x = e.getX();
            int y = e.getY();
            if (y>bufferHeight-12)
                currentValue = minValue;
            else if (y<12)
                currentValue = maxValue;
            else
                currentValue = ((double)((bufferHeight-24)-(y-12))/(double)(bufferHeight-24))*(maxValue-minValue);
            e.getComponent().repaint();
            if (adjustmentListeners.size() > 0) {
                int i=0; 
                AdjustmentEvent ae = new AdjustmentEvent ((Adjustable)e.getComponent(), AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED, AdjustmentEvent.TRACK, (int)currentValue);
                while (i<adjustmentListeners.size())
                    ((AdjustmentListener)adjustmentListeners.elementAt(i++)).adjustmentValueChanged (ae);
            }
        }
    }
}