package no.hig.okolloen.graphics.chart;

/**
 * The ValueScaler class is used to scale values so that graphs and charts look nice.
 * Initialize it with the max and min value and it will be ready for use.
 * Then ask it for labels and it will return an array of five strings with the max, min 
 * and three middle values to use for tags on your graphs and charts.
 * Use the getNormalizedValue to get a value between 0 and 1 that represents the 
 * scaled value given in respect to the labels.
 * If your data changes call the rescale with new max and min values and everything will be
 * recalculated.
 * This is the number 1 class I've always wanted when doing graphs and charts.
 * 
 * @author      Oivind Kolloen
 * @version     %I%, %G%
 * @since       rev 0.1
 */
public class ValueScaler {
    private double max;
    private double min;
    private double scaleFactor;
    private double offset;
    private int scaleType;
    public static final int SCALE_ONE_TO_MINUS_ONE = 0;
    public static final int SCALE_FIVE_TO_MINUS_FIVE = 1;
    public static final int SCALE_TWO_TO_MINUS_TWO = 2;
    public static final int SCALE_TEN_TO_ZERO = 3;
    public static final int SCALE_FOUR_TO_ZERO = 4;
    public static final int SCALE_TWO_TO_ZERO = 5;
    public static final int SCALE_ZERO_TO_MINUS_TEN = 6;
    public static final int SCALE_ZERO_TO_MINUS_FOUR = 7;
    public static final int SCALE_ZERO_TO_MINUS_TWO = 8;
    String labels[] = new String[5];

    /**
     *	Constructor for the ValueScaler class.
     *  @param max the maximum value in a given dataset.
     *  @param min the minumum value in a given dataset.
     */
    public ValueScaler (double max, double min) {
        rescale (max, min);
    }

    /**
     *	Recalibrates the internal data of the ValueScaler when your dataset changes its min and max values.
     *  @param max the maximum value in a given dataset.
     *  @param min the minumum value in a given dataset.
     */
    public void rescale (double max, double min) {
        if (max<0)
            max = 0;
        if (min>0)
            min = 0;
        this.max = max;
        this.min = min;
        double decimalCorrection = 1;
        double gap = max-min;
        while (gap>10) {
           gap /= (double)10;
           decimalCorrection *= (double)10;
        }
        while (gap<1) {
           gap *= (double)10;
           decimalCorrection /= (double)10;
        }
        if ((max>0)&&(min>=0)) {
           if (gap>4)
               scaleType = SCALE_TEN_TO_ZERO;
           else if (gap>2)
               scaleType = SCALE_FOUR_TO_ZERO;
           else
               scaleType = SCALE_TWO_TO_ZERO;
        } else if ((max<=0)&&(min<0)) {
           if (gap>4)
              scaleType = SCALE_ZERO_TO_MINUS_TEN;
           else if (gap>2)
              scaleType = SCALE_ZERO_TO_MINUS_FOUR;
           else
              scaleType = SCALE_ZERO_TO_MINUS_TWO;
        } else {
           if (gap>4)
              scaleType = SCALE_FIVE_TO_MINUS_FIVE;
           else if (gap>2)
              scaleType = SCALE_TWO_TO_MINUS_TWO;
           else
              scaleType = SCALE_ONE_TO_MINUS_ONE;
        }
        switch (scaleType) {
            case SCALE_ONE_TO_MINUS_ONE:   this.min=-1;  this.max=1;  break;
            case SCALE_FIVE_TO_MINUS_FIVE: this.min=-5;  this.max=5;  break;
            case SCALE_TWO_TO_MINUS_TWO:   this.min=-2;  this.max=2;  break;
            case SCALE_TEN_TO_ZERO:        this.min=0;   this.max=10; break;
            case SCALE_FOUR_TO_ZERO:       this.min=0;   this.max=4;  break;
            case SCALE_TWO_TO_ZERO:        this.min=0;   this.max=2;  break;
            case SCALE_ZERO_TO_MINUS_TEN:  this.min=-10; this.max=0;  break;
            case SCALE_ZERO_TO_MINUS_FOUR: this.min=-4;  this.max=0;  break;
            case SCALE_ZERO_TO_MINUS_TWO:  this.min=-2;  this.max=0;  break;
        }
        this.min *= decimalCorrection;
        this.max *= decimalCorrection;
        offset = 0-this.min;
        scaleFactor = (double)1/(this.max-this.min);
        updateScaleLabels();
    }

    private void updateScaleLabels () {
        switch (scaleType) {
            case SCALE_ONE_TO_MINUS_ONE:   labels[0] = "1"; labels[1] = "0.5"; labels[2] = "0"; labels[3] = "-0.5"; labels[4] = "-1"; break;
            case SCALE_FIVE_TO_MINUS_FIVE: labels[0] = "5"; labels[1] = "2.5"; labels[2] = "0"; labels[3] = "-2.5"; labels[4] = "-5"; break;
            case SCALE_TWO_TO_MINUS_TWO:   labels[0] = "2"; labels[1] = "1"; labels[2] = "0"; labels[3] = "-1"; labels[4] = "-2"; break;
            case SCALE_TEN_TO_ZERO:        labels[0] = "10"; labels[1] = "7.5"; labels[2] = "5"; labels[3] = "2.5"; labels[4] = "0"; break;
            case SCALE_FOUR_TO_ZERO:       labels[0] = "4"; labels[1] = "3"; labels[2] = "2"; labels[3] = "1"; labels[4] = "0"; break;
            case SCALE_TWO_TO_ZERO:        labels[0] = "2"; labels[1] = "1.5"; labels[2] = "1"; labels[3] = "0.5"; labels[4] = "0"; break;
            case SCALE_ZERO_TO_MINUS_TEN:  labels[0] = "0"; labels[1] = "-2.5"; labels[2] = "-5"; labels[3] = "-7.5"; labels[4] = "-10"; break;
            case SCALE_ZERO_TO_MINUS_FOUR: labels[0] = "0"; labels[1] = "-1"; labels[2] = "-2"; labels[3] = "-3"; labels[4] = "-4"; break;
            case SCALE_ZERO_TO_MINUS_TWO:  labels[0] = "0"; labels[1] = "-0.5"; labels[2] = "-1"; labels[3] = "-1.5"; labels[4] = "-2"; break;
        }
    }

    /**
     *	Returns an array of labels to use as markers on a graph/chart.
     *  @return String[] with new labels.
     */
    public String[] getScaleLabels () {
        return labels;
    }
        
    /**
     *	Returns an integer that corresponds to the scale range used internally.
     *  @return integer value corresponding to one of the public static final variables declared in the class.
     */
    public int getScaleType () {
        return scaleType;
    }

    /**
     *	Returns a double scaled according to the current max and min values.
     *  @return double value between 0 and 1.
     */
    public double getNormalizedValue (double value) throws RuntimeException {
        if (!((value<=max)&&(value>=min)))
            throw new RuntimeException ("Invalid value given to valueScaler : "+value+" is outside the range "+min+"<>"+max);
        return (value+offset)*scaleFactor;
    }

    /**
     *	Returns the current maximum limit of this ValueScaler.
     *  @return double value representing the maximum value that can be scaled with this object.
     */
    public double getMax () {
        return max;
    }

    /**
     *	Returns the current minumum limit of this ValueScaler.
     *  @return double value representing the minimum value that can be scaled with this object.
     */
    public double getMin () {
        return min;
    }


    /**
     *	Checks to see if the given value is a valid value for this ValueScaler.
     *  @param value is the value to be checked
     *  @return boolean true if the given value is larger than min and less than max.
     */
    public boolean isValidValue (double value) {
        return ((value<=max)&&(value>=min))?true:false;
    }
}